@@ -26,6 +26,19 @@
 #include <config.h>
 #endif
 
+/**
+ * SECTION:glade-property
+ * @Title: GladeProperty
+ * @Short_Description: An interface to properties on the #GladeWidget.
+ *
+ * Every object property of every #GladeWidget in every #GladeProject has
+ * a #GladeProperty to interface with, #GladeProperty provides a means
+ * to handle properties in the runtime environment.
+ * 
+ * A #GladeProperty can be seen as an instance of a #GladePropertyClass, 
+ * the #GladePropertyClass describes how a #GladeProperty will function.
+ */
+
 #include <stdio.h>
 #include <stdlib.h> /* for atoi and atof */
 #include <string.h>
@@ -93,21 +106,7 @@ glade_property_dup_impl (GladeProperty *template_prop, GladeWidget *widget)
 	return property;
 }
 
-void
-glade_property_reset_impl (GladeProperty *property)
-{
-	GLADE_PROPERTY_GET_KLASS (property)->set_value
-		(property, property->klass->def);
-}
-
-gboolean
-glade_property_default_impl (GladeProperty *property)
-{
-	return GLADE_PROPERTY_GET_KLASS (property)->equals_value
-		(property, property->klass->def);
-}
-
-gboolean
+static gboolean
 glade_property_equals_value_impl (GladeProperty *property,
 				  const GValue  *value)
 {
@@ -188,6 +187,29 @@ glade_property_update_prop_refs (GladeProperty *property,
 	}
 }
 
+static gboolean
+glade_property_verify (GladeProperty *property, const GValue *value)
+{
+	if (property->klass->packing)
+	{
+		if (property->widget->parent)
+			return glade_widget_adaptor_child_verify_property (property->widget->parent->adaptor,
+									   property->widget->parent->object,
+									   property->widget->object,
+									   property->klass->id,
+									   value);
+		else
+			return FALSE;
+	}
+	else
+	{
+		return glade_widget_adaptor_verify_property (property->widget->adaptor, 
+							     property->widget->object,
+							     property->klass->id,
+							     value);
+	}
+}
+
 static void
 glade_property_set_value_impl (GladeProperty *property, const GValue *value)
 {
@@ -198,8 +220,9 @@ glade_property_set_value_impl (GladeProperty *property, const GValue *value)
 
 #if 0
 	{
-		gchar *str = glade_property_class_make_string_from_gvalue
-			(property->klass, value);
+		gchar *str = glade_widget_adaptor_string_from_value
+			     (GLADE_WIDGET_ADAPTOR (property->klass->handle),
+			      property->klass, value);
 		g_print ("Setting property %s on %s to %s\n",
 			 property->klass->id,
 			 property->widget ? property->widget->name : "unknown", str);
@@ -217,13 +240,9 @@ glade_property_set_value_impl (GladeProperty *property, const GValue *value)
 	/* Check if the backend doesnt give us permission to
 	 * set this value.
 	 */
-	if (glade_property_superuser () == FALSE &&
-	    property->widget &&
+	if (glade_property_superuser () == FALSE && property->widget &&
 	    project && glade_project_is_loading (project) == FALSE &&
-	    glade_widget_adaptor_verify_property (property->widget->adaptor, 
-						  property->widget->object,
-						  property->klass->id,
-						  value) == FALSE)
+	    glade_property_verify (property, value) == FALSE)
 		return;
 	
 	/* save "changed" state.
@@ -325,8 +344,10 @@ glade_property_load_impl (GladeProperty *property)
 	
 	if (property->widget == NULL ||
 	    property->klass->packing ||
-	    property->klass->type != GPC_NORMAL)
+	    property->klass->ignore  ||
+	    !(property->klass->pspec->flags & G_PARAM_READABLE))
 		return;
+
 	object = glade_widget_get_object (property->widget);
 	oclass = G_OBJECT_GET_CLASS (object);
 	
@@ -334,121 +355,6 @@ glade_property_load_impl (GladeProperty *property)
 		g_object_get_property (object, property->klass->id, property->value);
 }
 
-static gboolean
-glade_property_write_impl (GladeProperty  *property, 
-			   GladeInterface *interface,
-			   GArray         *props)
-{
-	GladePropInfo         info  = { 0, };
-	GladeAtkActionInfo    ainfo = { 0, };
-	GList                *list;
-	gchar                *name, *value, **split, *tmp;
-	gint                  i;
-
-	if (!property->klass->save || !property->enabled)
-		return FALSE;
-
-	g_assert (property->klass->orig_def);
-	g_assert (property->klass->def);
-
-	/* Skip properties that are default
-	 * (by original pspec default) 
-	 */
-	if (glade_property_equals_value (property, property->klass->orig_def))
-		return FALSE;
-
-	/* we should change each '-' by '_' on the name of the property 
-         * (<property name="...">) */
-	if (property->klass->type != GPC_NORMAL)
-	{
-
-		tmp = (gchar *)glade_property_class_atk_realname (property->klass->id);
-		name = g_strdup (tmp);
-	}
-	else
-	{
-		name = g_strdup (property->klass->id);
-	}
-
-	/* convert the value of this property to a string */
-	if (property->klass->type == GPC_ACCEL_PROPERTY ||
-	    (value = glade_property_class_make_string_from_gvalue 
-	     (property->klass, property->value)) == NULL)
-		/* make sure we keep the empty string, also... upcomming
-		 * funcs that may not like NULL.
-		 */
-		value = g_strdup ("");
-	else
-	{
-		/* Escape the string so that it will be parsed as it should. */
-		tmp = value;
-		value = g_markup_escape_text (value, -1);
-		g_free (tmp);
-	}
-	
-	switch (property->klass->type)
-	{
-	case GPC_ATK_PROPERTY:
-		tmp = g_strdup_printf ("AtkObject::%s", name);
-		g_free (name);
-		name = tmp;
-		/* Dont break here ... */
-	case GPC_NORMAL:
-		info.name = glade_xml_alloc_propname(interface, name);
-		info.value = glade_xml_alloc_string(interface,  value);
-		
-		if (property->klass->translatable)
-		{
-			info.translatable = property->i18n_translatable;
-			info.has_context  = property->i18n_has_context;
-			if (property->i18n_comment)
-				info.comment = glade_xml_alloc_string
-					(interface, property->i18n_comment);
-		}
-		g_array_append_val (props, info);
-		break;
-	case GPC_ATK_RELATION:
-		if ((split = g_strsplit (value, GPC_OBJECT_DELIMITER, 0)) != NULL)
-		{
-			for (i = 0; split[i] != NULL; i++)
-			{
-				GladeAtkRelationInfo rinfo = { 0, };
-				rinfo.type   = glade_xml_alloc_string(interface, name);
-				rinfo.target = glade_xml_alloc_string(interface, split[i]);
-				g_array_append_val (props, rinfo);
-			}
-			g_strfreev (split);
-		}
-		break;
-	case GPC_ATK_ACTION:
-		ainfo.action_name = glade_xml_alloc_string(interface, name);
-		ainfo.description = glade_xml_alloc_string(interface, value);
-		g_array_append_val (props, ainfo);
-		break;
-	case GPC_ACCEL_PROPERTY:
-		for (list = g_value_get_boxed (property->value); 
-		     list; list = list->next)
-		{
-			GladeAccelInfo *accel = list->data;
-			GladeAccelInfo  accel_info = { 0, };
-
-			accel_info.signal    = glade_xml_alloc_string(interface, accel->signal);
-			accel_info.key       = accel->key;
-			accel_info.modifiers = accel->modifiers;
-
-			g_array_append_val (props, accel_info);
-		}
-		break;
-	default:
-		break;
-	}
-	
-	g_free (name);
-	g_free (value);
-
-	return TRUE;
-}
-
 static G_CONST_RETURN gchar *
 glade_property_get_tooltip_impl (GladeProperty *property)
 {
@@ -567,15 +473,12 @@ glade_property_klass_init (GladePropertyKlass *prop_class)
 
 	/* Class methods */
 	prop_class->dup                   = glade_property_dup_impl;
-	prop_class->reset                 = glade_property_reset_impl;
-	prop_class->def                   = glade_property_default_impl;
 	prop_class->equals_value          = glade_property_equals_value_impl;
 	prop_class->set_value             = glade_property_set_value_impl;
 	prop_class->get_value             = glade_property_get_value_impl;
 	prop_class->get_default           = glade_property_get_default_impl;
 	prop_class->sync                  = glade_property_sync_impl;
 	prop_class->load                  = glade_property_load_impl;
-	prop_class->write                 = glade_property_write_impl;
 	prop_class->get_tooltip           = glade_property_get_tooltip_impl;
 	prop_class->value_changed         = NULL;
 	prop_class->tooltip_changed       = NULL;
@@ -640,7 +543,6 @@ glade_property_klass_init (GladePropertyKlass *prop_class)
 
 }
 
-
 GType
 glade_property_get_type (void)
 {
@@ -668,306 +570,6 @@ glade_property_get_type (void)
 	return property_type;
 }
 
-/*******************************************************************************
-                        GladeInterface Parsing code
- *******************************************************************************/
-static GValue *
-glade_property_read_packing (GladeProperty      *property, 
-			     GladePropertyClass *pclass,
-			     GladeProject       *project,
-			     GladeChildInfo     *info,
-			     gboolean            free_value)
-{
-	GValue    *gvalue = NULL;
-	gint       i;
-	gchar     *id;
-
-	for (i = 0; i < info->n_properties; ++i)
-	{
-		GladePropInfo *pinfo = info->properties + i;
-		
-		id = glade_util_read_prop_name (pinfo->name);
-		
-		if (!strcmp (id, pclass->id))
-		{
-			gvalue = glade_property_class_make_gvalue_from_string
-				(pclass, pinfo->value, project);
-			
-			if (property)
-			{
-				glade_property_i18n_set_translatable
-					(property, pinfo->translatable);
-				glade_property_i18n_set_has_context
-					(property, pinfo->has_context);
-				glade_property_i18n_set_comment
-					(property, pinfo->comment);
-
-				property->enabled = TRUE;
-				
-				GLADE_PROPERTY_GET_KLASS (property)->set_value
-					(property, gvalue);
-			}
-
-			if (free_value)
-			{
-				g_value_unset (gvalue);
-				g_free (gvalue);
-			}
-
-			g_free (id);
-			break;
-		}
-		g_free (id);
-	}
-	return gvalue;
-}
-
-static GValue *
-glade_property_read_normal (GladeProperty      *property, 
-			    GladePropertyClass *pclass,
-			    GladeProject       *project,
-			    GladeWidgetInfo    *info,
-			    gboolean            free_value)
-{
-	GValue    *gvalue = NULL;
-	gint       i;
-	gchar     *id;
-
-	for (i = 0; i < info->n_properties; ++i)
-	{
-		GladePropInfo *pinfo = info->properties + i;
-		
-		id = glade_util_read_prop_name (pinfo->name);
-		
-		if (!strcmp (id, pclass->id))
-		{
-			if (property && glade_property_class_is_object (pclass))
-			{
-				/* we must synchronize this directly after loading this project
-				 * (i.e. lookup the actual objects after they've been parsed and
-				 * are present).
-				 */
-				g_object_set_data_full (G_OBJECT (property), 
-							"glade-loaded-object", 
-							g_strdup (pinfo->value), g_free);
-			}
-			else
-			{
-				gvalue = glade_property_class_make_gvalue_from_string
-					(pclass, pinfo->value, project);
-
-				if (property)
-					GLADE_PROPERTY_GET_KLASS
-						(property)->set_value (property, gvalue);
-
-				if (free_value)
-				{
-					g_value_unset (gvalue);
-					g_free (gvalue);
-				}
-			}
-			
-			if (property)
-			{
-				glade_property_i18n_set_translatable
-					(property, pinfo->translatable);
-				glade_property_i18n_set_has_context
-					(property, pinfo->has_context);
-				glade_property_i18n_set_comment
-					(property, pinfo->comment);
-
-				property->enabled = TRUE;
-			}
-
-			g_free (id);
-			break;
-		}
-		g_free (id);
-	}
-	return gvalue;
-}
-
-static GValue *
-glade_property_read_atk_prop (GladeProperty      *property, 
-			      GladePropertyClass *pclass,
-			      GladeProject       *project,
-			      GladeWidgetInfo    *info,
-			      gboolean            free_value)
-{
-	GValue    *gvalue = NULL;
-	gint       i;
-	gchar     *id;
-
-	for (i = 0; i < info->n_atk_props; ++i)
-	{
-		GladePropInfo *pinfo = info->atk_props + i;
-		
-		id = glade_util_read_prop_name (pinfo->name);
-		
-		if (!strcmp (id, pclass->id))
-		{
-			gvalue = glade_property_class_make_gvalue_from_string
-				(pclass, pinfo->value, project);
-
-			if (property)
-			{
-				glade_property_i18n_set_translatable
-					(property, pinfo->translatable);
-				glade_property_i18n_set_has_context
-					(property, pinfo->has_context);
-				glade_property_i18n_set_comment
-					(property, pinfo->comment);
-
-				property->enabled = TRUE;
-
-				GLADE_PROPERTY_GET_KLASS (property)->set_value
-					(property, gvalue);
-			}
-
-			if (free_value)
-			{
-				g_value_unset (gvalue);
-				g_free (gvalue);
-			}
-			
-			g_free (id);
-			break;
-		}
-		g_free (id);
-	}
-	return gvalue;
-}
-
-static GValue *
-glade_property_read_atk_relation (GladeProperty      *property, 
-				  GladePropertyClass *pclass,
-				  GladeProject       *project,
-				  GladeWidgetInfo    *info)
-{
-	const gchar *class_id;
-	gchar       *id, *string = NULL, *tmp;
-	gint         i;
-
-	for (i = 0; i < info->n_relations; ++i)
-	{
-		GladeAtkRelationInfo *rinfo = info->relations + i;
-			
-		id       = glade_util_read_prop_name (rinfo->type);
-		class_id = glade_property_class_atk_realname (pclass->id);
-
-		if (!strcmp (id, class_id))
-		{
-			if (string == NULL)
-				string = g_strdup (rinfo->target);
-			else
-			{
-				tmp = g_strdup_printf ("%s%s%s", string, 
-						       GPC_OBJECT_DELIMITER, rinfo->target);
-				string = (g_free (string), tmp);
-			}
-		}
-		g_free (id);
-	}
-
-	/* we must synchronize this directly after loading this project
-	 * (i.e. lookup the actual objects after they've been parsed and
-	 * are present).
-	 */
-	if (property)
-	{
-		g_object_set_data_full (G_OBJECT (property), "glade-loaded-object", 
-					g_strdup (string), g_free);
-	}
-
-	return NULL;
-}
-
-static GValue *
-glade_property_read_atk_action (GladeProperty      *property, 
-				GladePropertyClass *pclass,
-				GladeProject       *project,
-				GladeWidgetInfo    *info,
-				gboolean            free_value)
-{
-	GValue      *gvalue = NULL;
-	const gchar *class_id;
-	gchar       *id;
-	gint         i;
-
-	for (i = 0; i < info->n_atk_actions; ++i)
-	{
-		GladeAtkActionInfo *ainfo = info->atk_actions + i;
-			
-		id       = glade_util_read_prop_name (ainfo->action_name);
-		class_id = glade_property_class_atk_realname (pclass->id);
-
-		if (!strcmp (id, class_id))
-		{
-			/* Need special case for NULL values here ??? */
-			
-			gvalue = glade_property_class_make_gvalue_from_string 
-				(pclass, ainfo->description, project);
-				
-			if (property)
-				GLADE_PROPERTY_GET_KLASS
-					(property)->set_value (property, gvalue);
-
-			if (free_value)
-			{
-				g_value_unset (gvalue);
-				g_free (gvalue);
-			}
-			g_free (id);
-			break;
-		}
-		g_free (id);
-	}
-	return gvalue;
-}
-
-static GValue *
-glade_property_read_accel_prop (GladeProperty      *property, 
-				GladePropertyClass *pclass,
-				GladeProject       *project,
-				GladeWidgetInfo    *info,
-				gboolean            free_value)
-{
-	GValue      *gvalue = NULL;
-	GList       *accels = NULL;
-	gint         i;
-
-	for (i = 0; i < info->n_accels; ++i)
-	{
-		GladeAccelInfo *ainfo = info->accels + i;
-		
-		GladeAccelInfo *ainfo_dup = g_new0 (GladeAccelInfo, 1);
-		
-		ainfo_dup            = g_new0 (GladeAccelInfo, 1);
-		ainfo_dup->signal    = g_strdup (ainfo->signal);
-		ainfo_dup->key       = ainfo->key;
-		ainfo_dup->modifiers = ainfo->modifiers;
-
-		accels = g_list_prepend (accels, ainfo_dup);
-	}
-
-	gvalue = g_new0 (GValue, 1);
-	g_value_init (gvalue, GLADE_TYPE_ACCEL_GLIST);
-	g_value_take_boxed (gvalue, accels);
-
-	if (property)
-		GLADE_PROPERTY_GET_KLASS
-			(property)->set_value (property, gvalue);
-	
-	if (free_value)
-	{
-		g_value_unset (gvalue);
-		g_free (gvalue);
-	}
-
-	return gvalue;
-}
-
-
 /*******************************************************************************
                                      API
  *******************************************************************************/
@@ -977,30 +579,19 @@ glade_property_read_accel_prop (GladeProperty      *property,
  * @widget: The #GladeWidget this property is created for
  * @value: The initial #GValue of the property or %NULL
  *         (the #GladeProperty will assume ownership of @value)
- * @catalog_default: if specified; use any default value supplied
- *                   by the catalog; otherwise use the introspected default.
- *
  *
  * Creates a #GladeProperty of type @klass for @widget with @value; if
  * @value is %NULL, then the introspected default value for that property
- * will be used; unless otherwise specified by @catalog_default.
- *
- * Note that we want to use catalog defaults when creating properties for
- * any newly created #GladeWidget; but we want to stay with the introspected
- * defaults at load time (since the absence of the property who's default
- * has been overridden; is interpreted as explicitly set to the default
- * by the user).
+ * will be used.
  *
  * Returns: The newly created #GladeProperty
  */
 GladeProperty *
 glade_property_new (GladePropertyClass *klass, 
 		    GladeWidget        *widget, 
-		    GValue             *value,
-		    gboolean            catalog_default)
+		    GValue             *value)
 {
 	GladeProperty *property;
-	GValue        *def;
 	
 	g_return_val_if_fail (GLADE_IS_PROPERTY_CLASS (klass), NULL);
 
@@ -1015,12 +606,11 @@ glade_property_new (GladePropertyClass *klass,
 	
 	if (property->value == NULL)
 	{
-		def = catalog_default ? klass->def : klass->orig_def;
-		g_assert (def);
+		g_assert (klass->orig_def);
 		
 		property->value = g_new0 (GValue, 1);
-		g_value_init (property->value, def->g_type);
-		g_value_copy (def, property->value);
+		g_value_init (property->value, klass->orig_def->g_type);
+		g_value_copy (klass->orig_def, property->value);
 	}
 	return property;
 }
@@ -1039,6 +629,15 @@ glade_property_dup (GladeProperty *template_prop, GladeWidget *widget)
 	return GLADE_PROPERTY_GET_KLASS (template_prop)->dup (template_prop, widget);
 }
 
+static void
+glade_property_reset_common (GladeProperty *property, gboolean original)
+{
+	g_return_if_fail (GLADE_IS_PROPERTY (property));
+
+	GLADE_PROPERTY_GET_KLASS (property)->set_value
+		(property, (original) ? property->klass->orig_def : property->klass->def);
+}
+
 /**
  * glade_property_reset:
  * @property: A #GladeProperty
@@ -1048,8 +647,27 @@ glade_property_dup (GladeProperty *template_prop, GladeWidget *widget)
 void
 glade_property_reset (GladeProperty *property)
 {
-	g_return_if_fail (GLADE_IS_PROPERTY (property));
-	GLADE_PROPERTY_GET_KLASS (property)->reset (property);
+	glade_property_reset_common (property, FALSE);
+}
+
+/**
+ * glade_property_original_reset:
+ * @property: A #GladeProperty
+ *
+ * Resets this property to its original default value
+ */
+void
+glade_property_original_reset (GladeProperty *property)
+{
+	glade_property_reset_common (property, TRUE);
+}
+
+static gboolean
+glade_property_default_common (GladeProperty *property, gboolean orig)
+{
+	g_return_val_if_fail (GLADE_IS_PROPERTY (property), FALSE);
+	return GLADE_PROPERTY_GET_KLASS (property)->equals_value
+		(property, (orig)  ? property->klass->orig_def : property->klass->def);
 }
 
 /**
@@ -1061,8 +679,19 @@ glade_property_reset (GladeProperty *property)
 gboolean
 glade_property_default (GladeProperty *property)
 {
-	g_return_val_if_fail (GLADE_IS_PROPERTY (property), FALSE);
-	return GLADE_PROPERTY_GET_KLASS (property)->def (property);
+	return glade_property_default_common (property, FALSE);
+}
+
+/**
+ * glade_property_original_default:
+ * @property: A #GladeProperty
+ *
+ * Returns: Whether this property is at its original default value
+ */
+gboolean
+glade_property_original_default (GladeProperty *property)
+{
+	return glade_property_default_common (property, TRUE);
 }
 
 /**
@@ -1087,7 +716,7 @@ glade_property_equals_value (GladeProperty      *property,
  *
  * Returns: Whether this property is equal to the value provided
  */
-gboolean
+static gboolean
 glade_property_equals_va_list (GladeProperty *property, va_list vl)
 {
 	GValue   *value;
@@ -1274,83 +903,213 @@ glade_property_load (GladeProperty *property)
 /**
  * glade_property_read:
  * @property: a #GladeProperty or #NULL
- * @pclass: the #GladePropertyClass
  * @project: the #GladeProject
- * @info: a #GladeWidgetInfo struct or a #GladeChildInfo struct if
- *        a packing property is passed.
- * @free_value: Whether the return value should be freed after applying
- *              it to the property or if it should be returned in tact.
+ * @node: the #GladeXmlNode to read, will either be a 'widget'
+ *        node or a 'child' node for packing properties.
  *
- * Read the value and any attributes for @property from @info, assumes
+ * Read the value and any attributes for @property from @node, assumes
  * @property is being loaded for @project
  *
- * Returns: The newly created #GValue if successfull (and if @free_value == FALSE)
- *
  * Note that object values will only be resolved after the project is
  * completely loaded
  */
-GValue *
+void
 glade_property_read (GladeProperty      *property, 
-		     GladePropertyClass *pclass,
 		     GladeProject       *project,
-		     gpointer            info,
-		     gboolean            free_value)
+		     GladeXmlNode       *node)
 {
-	GValue *ret = NULL;
+	GladeXmlNode *prop;
+	GValue       *gvalue = NULL;
+	gchar        *id, *name, *value;
 
-	g_return_val_if_fail (pclass != NULL, FALSE);
-	g_return_val_if_fail (info != NULL, FALSE);
+	g_return_if_fail (GLADE_IS_PROPERTY (property));
+	g_return_if_fail (GLADE_IS_PROJECT (project));
+	g_return_if_fail (node != NULL);
 
-	if (pclass->packing)
+	/* This code should work the same for <packing> and <widget> */
+	if (!(glade_xml_node_verify_silent (node, GLADE_XML_TAG_PACKING) ||
+	      glade_xml_node_verify_silent
+	      (node, GLADE_XML_TAG_WIDGET (glade_project_get_format (project)))))
+		return;
+
+	for (prop = glade_xml_node_get_children (node); 
+	     prop; prop = glade_xml_node_next (prop))
 	{
-		ret = glade_property_read_packing 
-			(property, pclass, project, (GladeChildInfo *)info, free_value);
-	}
-	else switch (pclass->type)
-	{
-	case GPC_NORMAL:
-		ret = glade_property_read_normal
-			(property, pclass, project, (GladeWidgetInfo *)info, free_value);
-		break;
-	case GPC_ATK_PROPERTY:
-		ret = glade_property_read_atk_prop
-			(property, pclass, project, (GladeWidgetInfo *)info, free_value);
-		break;
-	case GPC_ATK_RELATION:
-		ret = glade_property_read_atk_relation
-			(property, pclass, project, (GladeWidgetInfo *)info);
-		break;
-	case GPC_ATK_ACTION:
-		ret = glade_property_read_atk_action
-			(property, pclass, project, (GladeWidgetInfo *)info, free_value);
-		break;
-	case GPC_ACCEL_PROPERTY:
-		ret = glade_property_read_accel_prop
-			(property, pclass, project, (GladeWidgetInfo *)info, free_value);
-		break;
-	default:
-		break;
-	}
 
-	return ret;
+		if (!glade_xml_node_verify_silent (prop, GLADE_XML_TAG_PROPERTY))
+			continue;
+
+		if (!(name = glade_xml_get_property_string_required
+		      (prop, GLADE_XML_TAG_NAME, NULL)))
+			continue;
+
+		/* Make sure we are working with dashes and
+		 * not underscores ... 
+		 */
+		id = glade_util_read_prop_name (name);
+		g_free (name);
+
+		if (!strcmp (id, property->klass->id))
+		{
+			if (!(value = glade_xml_get_content (prop)))
+			{
+				/* XXX should be glade_xml_get_content_required()... */
+				g_free (id);
+				break;
+			}
+
+			if (property && glade_property_class_is_object (property->klass))
+			{
+				/* we must synchronize this directly after loading this project
+				 * (i.e. lookup the actual objects after they've been parsed and
+				 * are present).
+				 */
+				g_object_set_data_full (G_OBJECT (property), 
+							"glade-loaded-object", 
+							g_strdup (value), g_free);
+			}
+			else
+			{
+				gvalue = glade_property_class_make_gvalue_from_string
+					(property->klass, value, project);
+
+				if (property) 
+				{
+					GLADE_PROPERTY_GET_KLASS
+						(property)->set_value (property, gvalue);
+				}
+
+				g_value_unset (gvalue);
+				g_free (gvalue);
+
+				/* If an optional property is specified in the
+				 * glade file, its enabled
+				 */
+				property->enabled = TRUE;
+			}
+
+			if (property)
+			{
+				gint translatable, has_context;
+				gchar *comment;
+
+				translatable = glade_xml_get_property_boolean
+					(prop, GLADE_TAG_TRANSLATABLE, FALSE);
+				has_context = glade_xml_get_property_boolean
+					(prop, GLADE_TAG_HAS_CONTEXT, FALSE);
+				comment = glade_xml_get_property_string
+					(prop, GLADE_TAG_COMMENT);
+
+				glade_property_i18n_set_translatable
+					(property, translatable);
+				glade_property_i18n_set_has_context
+					(property, has_context);
+				glade_property_i18n_set_comment
+					(property, comment);
+
+				g_free (comment);
+			}
+
+			g_free (value);
+			g_free (id);
+			break;
+		}
+		g_free (id);
+	}
 }
 
 
 /**
  * glade_property_write:
  * @property: a #GladeProperty
- * @interface: a #GladeInterface
- * @props: a GArray of #GladePropInfo
+ * @context: A #GladeXmlContext
+ * @node: A #GladeXmlNode
  *
- * Write this property to the GladeInterface metadata
+ * Write @property to @node
  */
-gboolean
-glade_property_write (GladeProperty *property, GladeInterface *interface, GArray *props)
+void
+glade_property_write (GladeProperty   *property,
+		      GladeXmlContext *context,
+		      GladeXmlNode    *node)
 {
-	g_return_val_if_fail (GLADE_IS_PROPERTY (property), FALSE);
-	g_return_val_if_fail (interface != NULL, FALSE);
-	g_return_val_if_fail (props != NULL, FALSE);
-	return GLADE_PROPERTY_GET_KLASS (property)->write (property, interface, props);
+	GladeXmlNode *prop_node;
+	GladeProject *project;
+	gchar *name, *value, *tmp;
+
+	g_return_if_fail (GLADE_IS_PROPERTY (property));
+	g_return_if_fail (node != NULL);
+
+	project = property->widget->project;
+
+	/* This code should work the same for <packing> and <widget> */
+	if (!(glade_xml_node_verify_silent (node, GLADE_XML_TAG_PACKING) ||
+	      glade_xml_node_verify_silent
+	      (node, GLADE_XML_TAG_WIDGET (glade_project_get_format(project)))))
+		return;
+
+	if (!property->klass->save || !property->enabled)
+		return;
+
+	g_assert (property->klass->orig_def);
+	g_assert (property->klass->def);
+
+	/* Skip properties that are default by original pspec default
+	 * (excepting those that specified otherwise).
+	 */
+	if (!(property->klass->save_always || property->save_always) &&
+	    glade_property_equals_value (property, property->klass->orig_def))
+		return;
+
+	/* Escape our string and save with underscores */
+	name = g_strdup (property->klass->id);
+	glade_util_replace (name, '-', '_');
+
+	/* convert the value of this property to a string */
+	if (!(value = glade_widget_adaptor_string_from_value
+	     (GLADE_WIDGET_ADAPTOR (property->klass->handle),
+	      property->klass, property->value)))
+		/* make sure we keep the empty string, also... upcomming
+		 * funcs that may not like NULL.
+		 */
+		value = g_strdup ("");
+	else
+	{
+		/* Escape the string so that it will be parsed as it should. */
+		tmp = value;
+		value = g_markup_escape_text (value, -1);
+		g_free (tmp);
+	}
+
+	/* Now dump the node values... */
+	prop_node = glade_xml_node_new (context, GLADE_XML_TAG_PROPERTY);
+	glade_xml_node_append_child (node, prop_node);
+
+	/* Name and value */
+	glade_xml_node_set_property_string (prop_node, GLADE_XML_TAG_NAME, name);
+	glade_xml_set_content (prop_node, value);
+
+	/* i18n stuff */
+	if (property->klass->translatable)
+	{
+		if (property->i18n_translatable)
+			glade_xml_node_set_property_string (prop_node, 
+							    GLADE_TAG_TRANSLATABLE, 
+							    GLADE_XML_TAG_I18N_TRUE);
+
+		if (property->i18n_has_context)
+			glade_xml_node_set_property_string (prop_node, 
+							    GLADE_TAG_HAS_CONTEXT, 
+							    GLADE_XML_TAG_I18N_TRUE);
+
+
+		if (property->i18n_comment)
+			glade_xml_node_set_property_string (prop_node, 
+							    GLADE_TAG_COMMENT, 
+							    property->i18n_comment);
+	}
+
+	g_free (name);
+	g_free (value);
 }
 
 /**
@@ -1547,6 +1306,41 @@ glade_property_get_sensitive (GladeProperty *property)
 	return property->sensitive;
 }
 
+/**
+ * glade_property_set_save_always:
+ * @property: A #GladeProperty
+ * @setting: the value to set
+ *
+ * Sets whether this property should be special cased
+ * to always be saved regardless of its default value.
+ * (used for some special cases like properties
+ * that are assigned initial values in composite widgets
+ * or derived widget code).
+ */
+void
+glade_property_set_save_always (GladeProperty      *property,
+				gboolean            setting)
+{
+	g_return_if_fail (GLADE_IS_PROPERTY (property));
+
+	property->save_always = setting;
+}
+
+/**
+ * glade_property_get_save_always:
+ * @property: A #GladeProperty
+ *
+ * Returns: whether this property is special cased
+ * to always be saved regardless of its default value.
+ */
+gboolean
+glade_property_get_save_always (GladeProperty      *property)
+{
+	g_return_val_if_fail (GLADE_IS_PROPERTY (property), FALSE);
+
+	return property->save_always;
+}
+
 void
 glade_property_set_enabled (GladeProperty *property, 
 			    gboolean       enabled)
